const { classes: Cc, interfaces: Ci, utils: Cu } = Components;

(function()
{
    const $ = require("ko/dom").window(window);
    const system = require("sdk/system");
    const koFile = require("ko/file");
    const prefs = require("ko/prefs");
    const mainWindow = require("ko/windows").getMain();
    const ko = mainWindow.ko;
    const log = require('ko/logging').getLogger("projectwizard");
    const fileService = Cc['@activestate.com/koFileService;1']
                .createInstance(Ci.koIFileService);
    const tbSvc     = Cc["@activestate.com/koToolbox2Service;1"]
                        .getService(Ci.koIToolbox2Service);

    var elems = {};
    var checkoutAsyncOp = null; // checkout operation
    var sccComponent = null;
    var dirChangedByUser = false;
    var cmdChangedByUser = false;
    var SCCSetByUser = false;
    var project;
    var sccNameDict =
    {
        bzr: 'Bazaar',
        cvs: 'CVS',
        git: "Git",
        hg: "Mercurial",
        p4: "Perforce",
        svn: "Subversion"
    };

    // Languages that have strong associations with one another
    var languageAssoc =
    {
        CSS: ["HTML", "HTML5", "Less", "SCSS"],
        Less: ["HTML", "HTML5", "CSS", "SCSS"],
        SCSS: ["HTML", "HTML5", "Less", "CSS"],
        HTML: ["HTML5", "JavaScript", "CSS", "SCSS", "Less"],
        HTML5: ["HTML", "JavaScript", "CSS", "SCSS", "Less"],
        JavaScript: ["HTML", "HTML5"],
        AngularJS: ["HTML", "HTML5", "JavaScript"],
        CoffeeScript: ["JavaScript", "Node.js"],
        LaravelBlade: ["PHP"],
        Python: ["Python3", "Django"],
        Python3: ["Python", "Django"],
        Django: ["Python", "Python3"]
    };

    var init = () =>
    {
        createElems();
        elems.$wizard.append(getPageStart().$element);
        elems.$wizard.append(getPageBootstrap().$element);
        elems.$wizard.append(getPageLoading().$element);
        $("window").append(elems.$wizard);
        updateWizardState();

        var finishBtn = $(elems.$wizard.element().getButton("finish"));
        finishBtn.attr("label", "Create");
        finishBtn.attr("accesskey", "C");

        elems.$wizard.element().addEventListener("wizardfinish", (event) => {onFinish(event);});
        elems.$wizard.element().canAdvance = false;
        
        elems.runCommand.on("input", () => cmdChangedByUser = true);

        // Work around XUL bug (yes, another one) where finish button is shown
        // instead of next button
        //elems.$wizard.element()._wizardButtons.onPageChange();

        sizeToContent();
        elems.$wizard.on("wizardnext", () => sizeToContent());
    };

    var sizeToContent = () =>
    {
        // eat a bag of  .. carrots.. XUL
        window.sizeToContent();
        setTimeout(() => window.sizeToContent(), 500);
    };
    
    var onFinish = (event) =>
    {
        // We want to force our own validation
        event.preventDefault();

        ko.projects.open(project.url);

        window.close();
        return true;
    };
    
    var createElems = () =>
    {
        elems.$wizard = $("<wizard>").attr("flex", "1");
        // where needed, extract in to further functions to generate list, add
        // event handlers, generate default values.
        elems.name = require("ko/ui/textbox").create();
        elems.name.on("input", onDirFieldElemInput);
        elems.dir = createDirFieldElem();
        elems.type = require("ko/ui/listbox").create({ attributes: { rows: 6 }});
        elems.language = $("<langlist>").attr({ "default": "None", "default-value": "" });
        elems.SCC = createSCCElem();
        elems.pathHeader = require("ko/ui/description").create({ attributes: { class: "fullwidth" } });
        elems.bootstrap = createBootstrapElem();
        elems.bootstrapSummary = require("ko/ui/description").create({ attributes: { class: "fullwidth" } });
        elems.bootstrapLabel = require("ko/ui/label").create("Template:");
        elems.bootstrapShowAll = showAllTemplatesElem();
        elems.runCommand = require("ko/ui/textbox").create();
        elems.newRepo = createNewRepoElem();
        elems.cloneLabel = require("ko/ui/label").create("Clone URL:");
        elems.repoUrl = require("ko/ui/textbox").create();
        elems.repoUrl.on("input", updateSCCStatus);
        elems.output = require("ko/ui/textbox").create({ attributes: { multiline: true, rows: 15, readonly: true } });
        elems.error = require("ko/ui/description").create({ attributes: { class: "state-error fullwidth", collapsed: true } });
    };

    var createSCCElem = () =>
    {
        var sccComponents = ko.scc.getAvailableSCCComponents();
        var field = require("ko/ui/menulist").create();
        field.addMenuItem({ attributes: { label: "None", value: "" }});

        for (let scc of ko.scc.getAvailableSCCComponents())
        {
            if(scc.component.getValue("supports_command", "checkout"))
            {
                field.addMenuItem({ attributes: { label: sccNameDict[scc.name], value: scc.name }});
            }
        }

        field.on("command", ()=>
        {
            SCCSetByUser = true;

            if( ! field.value())
            {
                sccComponent = null;
            }
            else
            {
                var sccName = field.value();
                for (let i=0; i < sccComponents.length; i++)
                {
                    if (sccName == sccComponents[i].name)
                    {
                        sccComponent = sccComponents[i].component;
                        break;
                    }
                }
            }

            updateSCCStatus();
        });

        return field;
    };

    var createBootstrapElem = () =>
    {
        var listbox = require("ko/ui/listbox").create({ attributes: { rows: 5 } });
        listbox.addListHeaders(['Name', 'Author', 'License']);
        listbox.addListCols(['', '', '']);

        return listbox;
    };

    var createNewRepoElem = () =>
    {
        var field = require("ko/ui/checkbox").create({ attributes: { label: "Initialize New Repository" }});
        field.on("command", updateSCCStatus);

        return field;
    };

    var onDirFieldElemInput = () =>
    {
        if (elems.dir && ! dirChangedByUser)
        {
            var path = koFile.join(
                    prefs.getStringPref("projects-dir"),
                    elems.name.value()
                );
            elems.dir.value(path);
            updateWizardState();
        }
    };

    // directory field
    // - will update Version Control field if applicable
    // - updates "Next" to "Finish" if directory already exists
    // -
    var createDirFieldElem = () =>
    {
        var field = require("ko/ui/filepath").create(
        {
            attributes:
            {
                value: prefs.getStringPref("projects-dir"),
                type: "autocomplete",
                filetype: "dir",
                autocompletesearch: "dirpath",
                autocompletepopup: "popupTextboxAutoComplete",
                maxrows: "10",
                enablehistory: "true",
                completeselectedindex: "true",
                tabscrolling: "true"
            }
        });
        var onInput = () =>
        {
            dirChangedByUser = true;
            updateWizardState();
        };

        field.on("input", onInput);
        return field;
    };

    var listTemplates = (all=false) =>
    {
        tbSvc.findToolsAsync("", 0, [], "folder_template", (code, results) =>
        {
            if (code !== 0)
            {
                log.error("findToolsAsync failed with code " + code);
            }

            results = results.sort((a,b) => { return a.name.localeCompare(b.name); });

            elems.bootstrap.removeAllItems();

            var noneItem = require("ko/ui/listitem").create({ attributes:
            {
                disabled: true,
                value: ""
            }});
            noneItem.addListCell("None");
            noneItem.addListCell("--");
            noneItem.addListCell("--");
            elems.bootstrap.addListItem(noneItem);

            var language = elems.language.value();
            var languages = [language];
            if (language in languageAssoc)
                languages = languages.concat(languageAssoc[language]);

            if ( ! languages.length)
                all = true;

            for (let tool of results)
            {
                if ( ! all && languages.indexOf(tool.koTool.getStringAttribute("language")) === -1)
                    continue;

                let listitem = require("ko/ui/listitem").create({ attributes: { value: tool.path_id } });
                listitem.addListCell(tool.name);
                listitem.addListCell(tool.koTool.getStringAttribute("author"));
                listitem.addListCell(tool.koTool.getStringAttribute("license"));
                elems.bootstrap.addListItem(listitem);
            }
        });
    };

    var showAllTemplatesElem = () =>
    {
        var toggle = () =>
        {
            if(elems.bootstrapShowAll.checked())
                listTemplates(true);
            else
                listTemplates();
        };

        var field = require("ko/ui/checkbox").create({ attributes: { label: "Show all templates" }});
        field.on("command", toggle);

        return field;
    };

    var updateTypeSelection = () =>
    {
        elems.type.removeAllItems();

        var language = elems.language.value();
        var languages = [language];
        if (language in languageAssoc)
            languages = languages.concat(languageAssoc[language]);

        var knownFrameworks = prefs.getPref("knownFrameworks");
        var frameworks = [];

        for (let id of knownFrameworks.getAllPrefIds().sort())
        {
            let framework = knownFrameworks.getPref(id);
            if (languages.indexOf(framework.getString("language")) == -1)
                continue;

            frameworks.push(id);
        }

        elems.type.addListItem(require("ko/ui/listitem").create({ attributes:
        {
            label: "None",
            value: ""
        }}));

        elems.type.addListItems(frameworks);
    };

    var getPageStart = () =>
    {
        var page = require("ko/ui/wizardpage").create({ attributes:
        {
            title:"Project Properties",
            pageid:"pageStart"
        }});

        var groupLocation = page.addGroupbox({ caption: "Location"});
        var groupType = page.addGroupbox({ caption: "Technology"});

        groupLocation.addRow([
            require("ko/ui/label").create("Name:"),
            elems.name
        ]);
        groupLocation.addRow([
            require("ko/ui/label").create("Location:"),
            elems.dir
        ]);
        
        elems.invalidPathLabel = require("ko/ui/label").create(
            "You entered the path to a file, please enter a path to a directory instead.",
            {
                class:"state-error"
            });
        groupLocation.addRow([require("ko/ui/row").create(),elems.invalidPathLabel]);

        groupType.addRow([
            require("ko/ui/label").create("Language:"),
            elems.language
        ]);

        groupType.addRow([
            require("ko/ui/label").create("Framework:"),
            elems.type
        ]);

        elems.language.on("command", () =>
        {
            elems.type.disable();
            if (elems.language.value !== "")
                elems.type.enable();
            updateTypeSelection();
        });

        elems.type.disable();
        updateTypeSelection();

        var canAdvance = () =>
        {
            elems.$wizard.element().canAdvance = !! elems.name.value() && !! elems.dir.value();
        };

        elems.name.on("input", () => canAdvance());
        elems.dir.on("input", () => canAdvance());

        elems.pageStart = page;

        page.on("pageShow", updateWizardState);

        return page;
    };

    var getPageBootstrap = () =>
    {
        var page = require("ko/ui/wizardpage").create({ attributes:
        {
            title:"Project Content",
            pageid:"pageBootstrap"
        }});
        page.addRow(elems.pathHeader);

        var groupSummary = page.addGroupbox();
        groupSummary.add(elems.bootstrapSummary);

        var group = page.addGroupbox({ caption: "Bootstrap Project"});

        group.addRow([
            require("ko/ui/label").create("Version Control:"),
            elems.SCC
        ]);

        group.addRow(elems.newRepo);
        group.addRow(
        [
            elems.cloneLabel,
            elems.repoUrl
        ]);

        group.add(require("ko/ui/separator").create());

        group.addRow(
        [
            elems.bootstrapLabel,
            elems.bootstrap
        ]);
        
        group.addRow(
        [
            elems.bootstrapShowAll
        ]);

        group.add(require("ko/ui/separator").create());

        group.addRow(
        [
            require("ko/ui/label").create("Run Command:"),
            elems.runCommand
        ]);

        group.addRow(
        [
            require("ko/ui/description").create(
                "This command will be ran in the project folder after it has been " +
                "created and after the template (if selected) has been imported."
            )
        ]);

        elems.repoUrl.on("input", () => updateSCCStatus());
       
        page.on("pageshow", onPageBootstrapShown);
        elems.pageBootstrap = page;
        
        return page;
    };
    
    var onPageBootstrapShown = () =>
    {
        var summary = `Creating ${elems.type.value()} project "${elems.name.value()}" in ${elems.dir.value()}`;
        elems.bootstrapSummary.value(summary);
        
        var type = elems.type.value();
        var knownFrameworks = prefs.getPref("knownFrameworks");
        var framework;

        for (let id of knownFrameworks.getAllPrefIds().sort())
        {
            if (id == type)
            {
                framework = knownFrameworks.getPref(id);
                break;
            }
        }

        if (framework && framework.hasPref('initCommand'))
        {
            elems.runCommand.value(framework.getStringPref('initCommand'));
        }

        listTemplates();
        updateSCCStatus();
    };
    
    var getPageLoading = () =>
    {
        var page = require("ko/ui/wizardpage").create({ attributes:
        {
            title: "Create Project",
            pageid: "pageLoading"
        }});

        var groupSummary = page.addGroupbox();
        groupSummary.add(elems.pathHeader);
        
        var group = page.addGroupbox({ caption: "Creating Project"});
        group.add(elems.output);

        group.add(require("ko/ui/spacer").create());
        group.add(elems.error);

        elems.pageLoading = page;

        page.on("pageshow", onPageLoadingShown);

        return page;
    };

    var onPageLoadingShown = () =>
    {
        var headerMsg = `Creating ${elems.type.value()} project "${elems.name.value()}" in ${elems.dir.value()}`;
        elems.pathHeader.value(headerMsg);

        window.creator(elems, (createdProject) =>
        {
            project = createdProject;
        }, log.error);
    };
    
    // If elems.dir.value() already exists and is already under SCC
    // control function disable selection of version control and jump to last
    // page when next is selected
    var updateWizardState = () =>
    {
        var path = elems.dir.value();
        var wizard = elems.$wizard.element();
        if(koFile.isDir(path) && ! koFile.isEmpty(path))
        {
            elems.invalidPathLabel.hide();
            elems.pageStart.element.next = elems.pageLoading.element.pageid;

            if (wizard.pageIndex === 0)
                wizard._wizardButtons._wizardButtonDeck.setAttribute("selectedIndex", 0); // finish button
        }
        else
        {
            if(koFile.isFile(path))
                elems.invalidPathLabel.show();
            // running on initialization seems to break the buttons
            // so don't do anything if it doesn't need to be done.
            if(elems.pageStart.element.next != elems.pageBootstrap.element.pageid)
            {
                elems.pageStart.element.next = elems.pageBootstrap.element.pageid;
                elems.pageBootstrap.element.next = elems.pageLoading.element.pageid;
            }
            
            if (wizard.pageIndex === 0)
                wizard._wizardButtons._wizardButtonDeck.setAttribute("selectedIndex", 1); // next button
        }
        
        if (wizard.pageIndex === 1)
            wizard._wizardButtons._wizardButtonDeck.setAttribute("selectedIndex", 0); // finish button

        updateSCCStatus();
        
        // More XUL workarounds .. cause why would this persist?
        wizard.canAdvance = ! wizard.canAdvance;
        wizard.canAdvance = ! wizard.canAdvance;
    };
    
    // Check if the directory field is under Version Control
    // Set the VersonControl field if it is, and is supported.
    var updateSCCStatus = (noDelay=false) =>
    {
        elems.newRepo.disable();
        elems.repoUrl.disable();
        elems.cloneLabel.disable();

        elems.bootstrapLabel.enable();
        elems.bootstrap.enable();
        elems.bootstrapShowAll.enable();

        if (["hg", "git"].indexOf(elems.SCC.value()) != -1)
        {
            elems.newRepo.enable();
        }
        else
        {
            elems.newRepo.checked(false);
        }

        if (elems.SCC.value() &&  ! elems.newRepo.checked())
        {
            elems.repoUrl.enable();
            elems.cloneLabel.enable();

            if ( ! cmdChangedByUser)
                elems.runCommand.value("");

            if (elems.repoUrl.value())
            {
                elems.bootstrapLabel.disable();
                elems.bootstrap.disable();
                elems.bootstrapShowAll.disable();
            }
        }
    };
    
    window.addEventListener("load", init);
})();